#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: sct_ec.tcl,v 2.43 2004/06/17 23:04:33 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hummel  09/07/98  created
#

#************************************************************************
#   NAME
#       sct_ec - fims remote control tcl script example
# 
#   SYNOPSIS
#       sct_ec
#
#   DESCRIPTION
#       this tcl script shows how to send command to the ESO Real Time Display 
#       (rtd) interface. Part of these commands are used by the FORS Observation
#       Block Editor (FOBE) 
#
#   PROCS 
#       proc init_ecosk - init external control of skycat
#                         looks if there is already a skycat sessions
#                         if there is not skycat session or a pure 
#                         rtd session start skycat and wait for the socket
#                         interface 
#       proc etcl cmd   - send a command to rtd
#       proc create_mask w 1 mode fors_menu 
#                       w = .skycat1
#                       mode = one of mos pmos img ipol lss hits 
#                       fors_menu = .skycat1.menubar.vlt-fors-1.m
#       proc delete_mask w 1 mode fors_menu 
#       proc action_mask method args 
#                        method is a public method of class FORS
#                        e.g. demo or
#                        config_mask "lss INS.SLIT.NAID lSlit0_3arcsec"
#
#   FILES
#        sct_ec.tcl
#
#   ENVIRONMENT
#        skycat tool 2.0.18 
#
#   RETURN VALUES
#
#   CAUTIONS
#
#   EXAMPLES
#        set sc .skycat1; set mode mos; set anchor .skycat1.menubar.vlt-fors-1.m
#        create_mask $sc 1 $mode $anchor
#        action mask config mask "mos INS.COLL.NAID COLL_SR+1" 
#        action_mask adjust_slit_length
#        action_mask config_mask "mos INS.GRIS.NAID GRIS_300I+11"
#        action_mask update_and_replot
#        delete_mask $sc 1 $mode $anchor
#
#   SEE ALSO
#
#   BUGS     
#
#------------------------------------------------------------------------
#


#--------------------------------------
# seqwish script von Peter Biereichel
#--------------------------------------
# to demonstrate the SkyCat Fims remote interface 
# start this script via
# seqWish fims/bin/sct_ec.tcl


#----------------
proc etcl {cmd} {
#----------------
   global rtd_fd
   return [send_to_rtd $rtd_fd remotetcl $cmd]
   }

#-------------------------
proc canvasCoords {x y } {
#-------------------------
   global img
   etcl "$img convert coords $x $y image cx0 cy0 canvas"
   set cx0 [etcl "set cx0"]
   set cy0 [etcl "set cy0"]
   return "$cx0 $cy0"
}



#---------------------------------------
proc init_ecosk {} {
#--------------------
# init external control of skycat

global rtd_fd sc ::env h_path int_path 

lappend auto_path /vlt/dflow/rtd/library

package require Rtd

set h_path $env(HOME)
set int_path $env(FIMSROOT)

set rtd_fd 0

# try if there is already a skycat running 
puts stdout [ set rtd_fd [connect_to_rtd] ]

if {$rtd_fd==0} {
   # nothing found, start fims 
   set message [exec $int_path/bin/fims.sh  0 &]
   } else {
   # something found
   # is it skycat or a remnant rtd session ?
   set sc [ etcl  "itcl::find objects -class ::skycat::SkyCat" ] 
   #puts stdout ">$sc<"
   if {[string match $sc "*not found*"]} {
        puts stdout "THIS IS NOT A SKYCAT SESSION"
        # this is not skycat, probably remote rtd 
        # purge the interface file
        exec rm $h_path/.rtd-remote
        exec touch $h_path/.rtd-remote
        set rtd_fd 0
        # start skycat explicitely
        set message [exec $int_path/bin/fims.sh  0 &]
        } 
   }

# try as along the variable is not defined 
 while {$rtd_fd==0} { 
       puts stdout [set rtd_fd [connect_to_rtd] ]
       after 5000 
       }

    # puts stdout "found : rtd_fd  = $rtd_fd"

    # set img [etcl ".rtd.image get_image"]
    # set canvas [etcl ".rtd.image get_canvas"]
    # puts stdout [ set name [ etcl  "tk appname" ] ];  
    # puts stdout [set ns [ etcl  "info namespace all skycat" ] ];           
    # puts stdout [set sc [ etcl  "utilNamespaceTail ::$ns" ] ]
    # puts stdout [set sc [ etcl  "info namespace qualifier ::$ns" ] ]
    # puts stdout [set sc [ etcl  "info classes" ] ]

puts stdout "SkyCat Pointer = [set sc [ etcl  "itcl::find objects -class ::skycat::SkyCat" ] ]"
puts stdout "SkyCat Port    = $rtd_fd"


# some  sub-pointers 
# set fors_menu [etcl ".sc component fors1 configure"]

set img    [etcl "$sc.image get_image"]
set canvas [etcl "$sc.image get_canvas"]
set anchor [etcl "$sc component fors2 cget -menu"]

puts stdout "img     = $img"
puts stdout "canvas  = $canvas"
puts stdout "fors_m  = $anchor"
puts stdout "init_ecosk end\n------------------------"
return $anchor 
#------------------------------------------
}


# peters example application 
# set cfrom [canvasCoords 10 20]
# puts stdout "cfrom  = $cfrom"
# set cto [canvasCoords 400 500]
# puts stdout "cto = $cto"
# etcl "$canvas create line $cfrom $cto -tags Bernard -width 5 -fill white"


  
#----------------------------------------
proc create_mask { w f_number mode fors_menu } {
#-----------------------------------------
     # Create a mask
     # f_on { w f_number mode m }
     # a) wrapper f_on, also updating the menu buttons 
     # ::fors::FORS::FORSInit canvas image modus 1 0 
     puts stdout [etcl "::f_on $w $f_number $mode $fors_menu" ] 

     #  LSS 
     #  ---
     # disable MOS instrument configuarion buttons 
     if {[string compare $mode "lss"]==0} {
            puts stdout [etcl "$fors_menu.layout entryconfigure 1 -state disabled"] 
            puts stdout [etcl "$fors_menu.layout entryconfigure 2 -state disabled"]
            puts stdout [etcl "$fors_menu.layout entryconfigure 4 -state disabled"]
       } elseif {[string compare $mode "hits"]==0} {
            puts stdout [etcl "$fors_menu.layout entryconfigure 1 -state disabled"] 
            puts stdout [etcl "$fors_menu.layout entryconfigure 2 -state disabled"]
            puts stdout [etcl "$fors_menu.layout entryconfigure 3 -state disabled"]
            } else {
     # MOS/PMOS/IMG/PIMG
     # -----------------
     # disable LSS instrument configuration button 
     puts stdout [etcl "$fors_menu.layout entryconfigure 3 -state disabled" ] 
     # disable HITS instrument configuration button 
     puts stdout [etcl "$fors_menu.layout entryconfigure 4 -state disabled" ] 
     }

     # Filter    
     puts stdout [etcl "$fors_menu.layout entryconfigure 6 -state disabled" ] 
     # Grism      
     puts stdout [etcl "$fors_menu.layout entryconfigure 7 -state disabled" ] 
     # Collimator
     puts stdout [etcl "$fors_menu.layout entryconfigure 8 -state disabled" ] 
     # Delete Mask 
     puts stdout [etcl "$fors_menu        entryconfigure 4 -state disabled" ] 
     # Save Mask 
     puts stdout [etcl "$fors_menu        entryconfigure 7 -state disabled" ] 
     }


#----------------------------------------
proc delete_mask { w f_number mode fors_menu } {
#-----------------------------------------
    # Delete a mask
    # b) wrapper f_on, also updating the menu buttons
    # ::fors::FORS::FORSInit canvas image modus 1 0
puts stdout [etcl "::f_off $w $f_number  $mode $fors_menu"]
}

#---------------------------------------
# general usage 
# ::namespace::class::instance method 
# particular usage 
# ::fors::FORS::my_mask demo
#---------------------------------------


#----------------------------------------
proc action_mask { method args } {
#---------------------------------
#use public methods of FORS class 
return [etcl "::fors::FORS::my_mask $method $args"]
}


# --------------------------------
#  S T A R T
# --------------------------------
# definitions finished
# now do something 
# init

puts stdout [set anchor [eval init_ecosk] ]
puts stdout "extcont: anchor=$anchor"


#++++++++++++++++++++++++++++++++++++++++++++++++++++++
#------------------------------------------------------
#======================================================
#              Now some examples
#======================================================
#------------------------------------------------------
#++++++++++++++++++++++++++++++++++++++++++++++++++++++



puts stdout "\n\n TEST the current SkyCat FIMS Version\n"

puts stdout " TEST  001 : load test image PREP_DIR/TEST_FIMS/FORS1.fits"
set msg [etcl "::skycat::SkyCat::load_image $h_path/.fims/PREP/TEST_FIMS/FORS1.fits " ]
puts stdout " TEST  001 : $msg"

set img    [etcl "$sc.image get_image"]
set canvas [etcl "$sc.image get_canvas"]
# delete graphics from canvas 
set msg [etcl "$canvas delete" ]


puts stdout " TEST  002 : get WCS keywords from test image"
set eqx [ lindex [etcl "$img wcsset" ] 8]
set cr1 [etcl "$img fits get CRPIX1" ]
set cr2 [etcl "$img fits get CRPIX2" ]
set cv1 [etcl "$img fits get CRVAL1" ]
set cv2 [etcl "$img fits get CRVAL2" ]

puts stdout " TEST  002 : equinox     $eqx"
puts stdout " TEST  002 : refpixel at $cr1 $cr2"
puts stdout " TEST  002 : refwcs   at $cv1 $cv2"

set string "{wcs $eqx}"
# puts stdout " TEST  002 : $string"
set msg [etcl "$img convert coords $cr1 $cr2 image cx0 cy0 $string" ]
puts stdout " TEST  002 : $msg"
set cx0 [etcl "set cx0"]
set cy0 [etcl "set cy0"]
puts stdout " TEST  002 : refwcs   at $cx0 $cy0"

# puts stdout " TEST  003 : scale image "
# set msg [etcl "$img scale -4 -4" ]
# set msg [etcl "$img update" ]
# puts stdout " TEST  003 : $msg "

# puts stdout " TEST  004 : cut image "
# set msg [etcl "$img cut 400 800" ]
# set msg [etcl "$img update" ]
# puts stdout " TEST  004 : $msg "

puts stdout " TEST  005 : delete mask"
delete_mask $sc 1 mos $anchor

puts stdout " TEST  006 : delete canvas drawings"
set msg [etcl "$canvas delete" ]
puts stdout " TEST  006 : $msg "

puts stdout " TEST  007 : create MOS-2"
create_mask $sc 1 mos $anchor


#------------
# method demo 
#------------
 set mode "mos"
 foreach mode {lss mos hits img echs mxu} {
    puts stdout "delete_mask $sc 1 $mode $anchor"
    delete_mask $sc 1 $mode $anchor

    puts stdout "create_mask $sc  1 $mode $anchor"
    create_mask $sc 1 $mode $anchor

    puts stdout [set a [action_mask configure -label 0] ]
    puts stdout [set a [action_mask configure -label 1] ]
    puts stdout [set a [action_mask demo] ]
    }


#--------------------------------
# method config mask : Collimator 
#--------------------------------
# create_mask $sc $mode $anchor
# set mode mos; 
# set key INS.COLL.NAID; 
# 
#     delete_mask $sc 1 $mode $anchor
#     puts stdout "create_mask $sc  1 $mode $anchor"
#     create_mask $sc 1 $mode $anchor
# 
#     
#  for {set i 1 } {$i<6} {incr i 1} {
#    set val COLL_HR+2;
#    puts stdout [ set rein "$mode $key $val" ]
#    action_mask config_mask  $rein
#    # action_mask adjust_slit_length
#    puts stdout [ set a [action_mask update_and_replot] ]
# 
#    set val COLL_SR+1
#    puts stdout [ set rein "$mode $key $val" ]
#    action_mask config_mask  $rein
#    # action_mask adjust_slit_length
#    puts stdout [ set a [action_mask update_and_replot] ]
#  }
# 
#    set val COLL_HR+2;
#    puts stdout [ set rein "$mode $key $val" ]
#    action_mask config_mask  $rein
#    # action_mask adjust_slit_length
#    puts stdout [ set a [action_mask update_and_replot] ]


#--------------------------------
# method config mask : GRISM 
#--------------------------------
#  foreach gris { GRIS_600B+12 GRIS_600R+14 GRIS_600I+15 GRIS_600z+16 GRIS_300V+10 \
#                 GRIS_300I+11 GRIS_150I+17} {
#          puts [set rein "mos INS.GRIS.NAID $gris" ]
#          action_mask config_mask  $rein
#          # action_mask adjust_slit_length
#          puts stdout [ set a [action_mask reset_mask] ]
#          }

#--------------------------------
# method config mask : OrderSeparation Filters 
#--------------------------------
# set osf GG375+30; #  (or GG435+31 OG590+32)
# set rein "mos INS.FILT.NAID $osf"    
# action_mask config_mask  $rein
# puts stdout [ set a [action_mask update_and_replot] ]

#--------------------------------
# method config mask : UserDefined wavelength region 
#--------------------------------
# set rein "mos INS.WAVELENGTH.MIN 500.0"
# saction_mask config_mask  $rein
# saction_mask adjust_slit_length
# sputs stdout [ set a [action_mask reset_mask] ]

#--------------------------------
# method save_TPX : save Target Acquisition File 
#--------------------------------
# s puts stdout [ set a [action_mask save_TPX micky_mouse.targ ] ]


#--------------------------------
# method config mask : LSS/HITS
#--------------------------------
# delete_mask  $sc 1 $mode $anchor
# create_mask  $sc 1 hits  $anchor
# puts stdout [set a [action_mask configure -superuser 1] ]
#  after 2000; update

#  set rein "hits INS.SLIT.NAID lSlit0_3arcsec"
#  action_mask config_mask  $rein
#  puts stdout [ set a [action_mask update_mask] ]
#  puts stdout [ set a [action_mask reset_mask] ]
#  after 2000; update
# 
#  set rein "hits INS.SLIT.NAID lSlit0_4arcsec"
#  action_mask config_mask  $rein
#  puts stdout [ set a [action_mask update_mask] ]
#  puts stdout [ set a [action_mask reset_mask] ]
#  after 2000; update
# 
#  set rein "hits INS.SLIT.NAID lSlit0_5arcsec"
#  action_mask config_mask  $rein
#  puts stdout [ set a [action_mask update_mask] ]
#  puts stdout [ set a [action_mask reset_mask] ]
#  after 2000; update
# 
#--------------------------------
# method config mask : LSS
#--------------------------------
#delete_mask  $sc 1 $mode $anchor
#create_mask  $sc 1 mos  $anchor
#after 2000; update


puts stdout "--->>> sct_ec end <<< ---"
# delete_mask $sc 1 $mode $anchor

